<?php

/**
 * //
 */

namespace DeubertIt\Healthchecks;

use Composer\Composer;
use Composer\Script\Event;
use Composer\EventDispatcher\EventSubscriberInterface;
use Composer\IO\IOInterface;
use Composer\Plugin\PluginInterface;
use Composer\Installer\PackageEvent;
use Composer\Package\PackageInterface;

class Healthchecks implements PluginInterface, EventSubscriberInterface
{
    const PACKAGE_NAME = 'deubertit/composer-healthchecks';

    const EXTRA_PUBLIC_PATH = 'dit-healthchecks-public-path';

    const HEALTHCHECKS_SRC_FILENAME = 'healthcheck.dist.php';
    const HEALTHCHECKS_DEST_FILENAME = 'healthcheck.php';
    const HEALTHCHECK_PLACEHOLDER_AUTOLOAD = '##healthchecks-autoload-path##';
    const HEALTHCHECK_PLACEHOLDER_COMPOSER_FILE = '##composer-filename##';

    /**
     * @var Composer
     */
    protected $_composer;

    /**
     * @var IOInterface
     */
    protected $_io;

    /**
     * @var string
     */
    protected $_publicPath;

    /**
     * @return  array       Callbacks list for composer events
     */
    public static function getSubscribedEvents()
    {
        return [
            'post-package-install' => [
                ['onPackageAction', 0]
            ],
            'post-package-update' => [
                ['onPackageAction', 0]
            ]
        ];
    }

    /**
     * @param   Composer            $composer
     * @param   IOInterface         $io
     */
    public function activate(Composer $composer, IOInterface $io)
    {
        $this->_composer = $composer;
        $this->_io = $io;

        $extra = $this->_composer->getPackage()->getExtra();

        if (!isset($extra[self::EXTRA_PUBLIC_PATH]) 
            || empty($extra[self::EXTRA_PUBLIC_PATH]))
        {
            $this->_io->writeError(sprintf(
                '  --- %s: extra.%s value is mandatory!', 
                self::PACKAGE_NAME,
                self::EXTRA_PUBLIC_PATH
            ));

            return;
        }

        $this->_publicPath = rtrim(getcwd() . '/' . $extra[self::EXTRA_PUBLIC_PATH], '/');
    }

    /**
     * @param  PackageEvent         $event
     */
    public function onPackageAction(PackageEvent $event)
    {
        if (empty($this->_publicPath))
        {
            // module is not configured properly
            // error message will be shown before in self::activate()
            return;
        }

        if (method_exists($event->getOperation(), 'getPackage'))
        {
            // on initial installation
            $package = $event->getOperation()->getPackage();
        }
        else
        {
            // on updates
            $package = $event->getOperation()->getTargetPackage();
        }

        if ($package->getName() === self::PACKAGE_NAME)
        {
            $composerConfig = $this->_composer->getConfig();

            $packageDir = $composerConfig->get('vendor-dir') . '/' . self::PACKAGE_NAME;
            $srcHealthchecksFile = $packageDir . '/' . self::HEALTHCHECKS_SRC_FILENAME;

            if (!is_file($srcHealthchecksFile) || !($fileContent = file_get_contents($srcHealthchecksFile)))
            {
                $this->_io->writeError(sprintf(
                    '  --- %s: source healthcheck file could not be found in %s!',
                    self::PACKAGE_NAME,
                    $srcHealthchecksFile
                ));

                return;
            }

            $fileContent = str_replace([
                self::HEALTHCHECK_PLACEHOLDER_AUTOLOAD,
                self::HEALTHCHECK_PLACEHOLDER_COMPOSER_FILE
            ], [
                $packageDir . '/autoload.php',
                $composerConfig->getConfigSource()->getName()
            ], $fileContent);

            $destHealthchecksFile = $this->_publicPath . '/' . self::HEALTHCHECKS_DEST_FILENAME;

            if (is_file($destHealthchecksFile) && !unlink($destHealthchecksFile))
            {
                $this->_io->writeError(sprintf(
                    '  --- %s: destination healthcheck file is already exists %s and can not be deleted',
                    self::PACKAGE_NAME,
                    $destHealthchecksFile
                ));

                return;
            }

            if (!is_dir($this->_publicPath))
            {
                mkdir($this->_publicPath, 0777, true);
            }

            if ( ! @file_put_contents($destHealthchecksFile, $fileContent))
            {
                $this->_io->writeError(sprintf(
                    '  --- %s: destination healthcheck file could not be created in %s!',
                    self::PACKAGE_NAME,
                    $destHealthchecksFile
                ));

                return;
            }

            $this->_io->write('  --- healthckecks script successfully installed to ' . $destHealthchecksFile);
        }
    }
}
