<?php

namespace DeubertIt\Healthchecks;

class Requirements
{
    /**
     * @var array
     */
    protected $_config;

    /**
     * @param   string          $composerFilename
     */
    public function __construct($composerFilename)
    {
        if (!is_file($composerFilename))
        {
            throw new \Exception('Composer config not found in ' . $composerFilename);
        }

        $configContent = file_get_contents($composerFilename);
        $this->_config = json_decode($configContent, true);

        if (is_null($this->_config))
        {
            throw new \Exception('Invalid composer config in ' . $composerFilename);
        }
    }

    /**
     * @throws  Exception
     */
    public function checkPhpRequirements()
    {
        $requiredVersion = $this->_getRequiredVersion('php');
        $currentVersion = phpversion();

        if (is_null($requiredVersion))
        {
            // php version is not specified in the config
            return;
        }

        if (!$this->_isVersionOk($requiredVersion, $currentVersion))
        {
            throw new \Exception(sprintf(
                'Invalid php version installed %s (%s required)',
                $currentVersion, 
                $requiredVersion
            ));
        }
    }

    /**
     * @throws  Exception
     */
    public function checkExtensionsRequirements()
    {
        $requiredExtensions = $this->_getRequiredExtensions();

        if (empty($requiredExtensions))
        {
            // not specified in the config
            return;
        }

        foreach ($requiredExtensions as $extension)
        {
            // @todo probably check first with get_loaded_extensions()

            $requiredVersion = $this->_getRequiredVersion("ext-$extension");
            $currentVersion = phpversion($extension);

            if ($currentVersion === false)
            {
                throw new \Exception(sprintf(
                    'Extension "%s" is not installed (%s required)',
                    $extension,
                    $requiredVersion
                ));
            }

            if (!$this->_isVersionOk($requiredVersion, $currentVersion))
            {
                throw new \Exception(sprintf(
                    'Invalid "%s" extension version installed %s (%s required)',
                    $extension,
                    $currentVersion, 
                    $requiredVersion
                ));
            }
        }
    }

    protected function _getRequiredExtensions()
    {
        if (!isset($this->_config['require']) || empty($this->_config['require']))
        {
            return null;
        }

        $extensions = [];

        foreach (array_keys($this->_config['require']) as $name)
        {
            if (strpos($name, 'ext-') === 0)
            {
                $extensions[] = substr($name, 4); // remove 'ext-' prefix
            }
        }

        return $extensions;
    }

    /**
     * @param   string          $key
     * @returns mixed
     */
    protected function _getRequiredVersion($key)
    {
        return isset($this->_config['require']) && isset($this->_config['require'][$key])
            ? $this->_config['require'][$key]
            : null;
    }

    /**
     * @param   string          $required
     * @param   string          $current
     * @returns boolean
     */
    protected function _isVersionOk($required, $current)
    {
        if (is_numeric($required[0]))
        {
            // strict version number
            return version_compare($current, $required) === 0;
        }

        if ($required === '*')
        {
            // any version is okay
            return !empty($current);
        }

        $supportedConstraints = ['<=', '<', '>=', '>'];

        foreach ($supportedConstraints as $sign)
        {
            if (strpos($required, $sign) === 0)
            {
                $requiredClean = substr($required, strlen($sign));
                return version_compare($current, $requiredClean, $sign);
            }
        }

        throw new \Exception(sprintf(
            'Unsupported version requirement "%s" for %s',
            $requiredVersion,
            $extension
        ));
    }
}
